<?php

namespace Drupal\devel_tomsaw\Commands;

use Drush\Attributes as CLI;
use Drush\Commands\DrushCommands;
use Drupal\Core\Database\Connection;
use Drupal\field\Entity\FieldStorageConfig;
use Symfony\Component\Console\Input\InputOption;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\memcache\Driver\MemcacheDriverFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drush\Commands\field\EntityTypeBundleAskTrait;
use Drush\Commands\field\EntityTypeBundleValidationTrait;

/**
 * A Drush commandfile.
 *
 * In addition to this file, you need a drush.services.yml
 * in root of your module, and a composer.json file that provides the name
 * of the services file to use.
 */
final class DevelTomSawCommands extends DrushCommands
{
  use EntityTypeBundleAskTrait;
  use EntityTypeBundleValidationTrait;

  public function __construct(
    protected EntityTypeManagerInterface $entityTypeManager,
    protected ModuleHandlerInterface $moduleHandler,
    protected MemcacheDriverFactory $memcacheFactory,
    protected Connection $database
  ) {
  }

  /**
   * Flush the memcache
   */
  #[CLI\Command(name: 'cache:flush-memcache', aliases: ['cfm'])]
  public function flushMemcache()
  {
    if (!$this->moduleHandler->moduleExists('memcache')) {
      $this->logger()->warning("memcache module not installed");
      return;
    }

    $this->memcacheFactory->get('default')->flush();
    $this->logger()->success("Memcache flushed");
  }

  #[CLI\Command(name: 'devel:change-textfield-max-length', aliases: ['dctml'])]
  #[CLI\Argument(name: 'entityType', description: 'Entity type')]
  // #[CLI\Option(name: 'field-name', description: 'A unique machine-readable name containing letters, numbers, and underscores.')]
  // #[CLI\Argument(name: 'new-length', description: 'Length')]
  public function changeTextFieldMaxLength(?string $entityType = null, ?string $fieldName, ?int $newLength, array $options = [
      'format' => 'table',
    ]
  ): void {
    $this->input->setArgument('entityType', $entityType = $entityType ?? $this->askEntityType());
    $this->validateEntityType($entityType);

    $schema = $this->database->schema();
    $table = $entityType . '__' . $fieldName;
    $table_revision = $entityType . '_revision__' . $fieldName;
    $new_field = [
      'type' => 'varchar',
      'length' => $newLength,
    ];
    $col_name = $fieldName . '_value';

    try {
      $schema->changeField($table, $col_name, $col_name, $new_field);

      if ($schema->tableExists($table_revision)) {
        $schema->changeField($table_revision, $col_name, $col_name, $new_field);
      }

      $storage_key = "$entityType.field_schema_data.$fieldName";
      $storage_schema = \Drupal::keyValue('entity.storage_schema.sql');
      $field_schema = $storage_schema->get($storage_key);
      $field_schema[$table]['fields'][$col_name]['length'] = $newLength;

      if ($schema->tableExists($table_revision)) {
        $field_schema[$table_revision]['fields'][$col_name]['length'] = $newLength;
      }

      $storage_schema->set($storage_key, $field_schema);

      \Drupal::configFactory()
        ->getEditable("field.storage.$entityType.$fieldName")
        ->set('settings.max_length', $newLength)
        ->save(TRUE);

      FieldStorageConfig::loadByName($entityType, $fieldName)->save();
    } catch (\Exception $e) {
      \Drupal::logger('system')->error(
        'Failed to update field length for @entity_type.@field: @message',
        [
          '@entity_type' => $entityType,
          '@field' => $fieldName,
          '@message' => $e->getMessage(),
        ]
      );
    }
  }
}
